package model;

import java.io.*;
import java.util.*;

import listener.LayerChangeEvent;
import listener.LayerListener;

import ui.*;
import utils.*;

public class LayerList {
	// public static final int BATSU = 11;
	private int layer;
	private List<LayerListener> listeners = new ArrayList<LayerListener>();

	private ArrayList<Board> maps = new ArrayList<Board>();

	LayerList(final int width, final int height) {
		newMaps(width, height);
	}

	public void saveData(File file) {
		// TODO f[^Z[u
		// Z[u͔ẑ܂ܔo`ōs
		// t@C̍쐬
		String fileName = file.getPath(); // tpX̃t@C𓾂
		String lowerName = fileName.toLowerCase();
		if (!lowerName.endsWith(".dat")) {
			fileName += ".dat";
		}
		saveFile(new File(fileName)); // Z[ũ\bhĂяo
	}

	private void saveFile(File file) {
		// TODO zo
		// String data = maps.get(layer).getData();
		String data = maps.get(0).getData();
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(file));
			writer.write(data); // f[^
			writer.close();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/** t@CI[v */
	public void loadDat(File file) {
		maps.get(layer).loadDat(file);
	}

	/** ׂẴV} */
	public void deleteAllIslands() {
		maps.get(layer).deleteAllIslands();
		notifyToLayerListeners();
	}

	public void increaseLayer() {
		layer++;
		if (layer == maps.size())
//			maps.add(new Board(getWidth(), getHeight()));
			maps.add(new Board(getColumnCount(), getRowCount()));
		notifyToLayerListeners();
	}

	public void decreaseLayer() {
		layer--;
		notifyToLayerListeners();
	}

	protected void notifyToLayerListeners() {
		for (LayerListener listener : listeners) {
			listener.layerChanged(new LayerChangeEvent(this));
		}
	}

	public int getLayer() {
		return layer;
	}

	public void addLayerListener(LayerListener layerListener) {
		listeners.add(layerListener);
	}

	public void deleteBamboo(int xMin, int xMax, int yMin, int yMax) {
		BoardUtils.doForSquare(maps.get(layer), new EachCellOperation() {
			public boolean operation(int row, int column, Board board) {
				// TODO V}
				return BoardUtils.deleteBambooCell(board.getCell(row, column));
			}
		}, xMin, yMin, xMax - xMin + 1, yMax - yMin + 1);
	}

	public void setMap(int[][] map, int xstart, int width, int ystart,
			int height) {
		maps.get(layer).setMap(map, xstart, width, ystart, height);
		notifyToLayerListeners();
	}

	public void encode(File file, String format, SaveController saveController,
			ApplicationView view) {
		maps.get(layer).encode(file, format, saveController, view);
	}

	public void saveAtf(PrintWriter writer, int x, int y, int saveWidth,
			int saveHeight) {
		maps.get(layer).saveAtf(writer, x, y, saveWidth, saveHeight);
	}

	public int getMap(int layer, int i, int j) {
		return maps.get(layer).getMap(i, j);
	}

	public int getMapRowColumn(int layer, int row, int column) {
		return maps.get(layer).getMapRowColumn(row, column);
	}

	public int getLayerSize() {
		return maps.size();
	}

	public void setCross(int x, int y) {
		CellUtils.setCross(maps.get(layer).getCell(x, y));
		notifyToLayerListeners();
	}

	public String getURL(int x, int y, int saveWidth, int saveHeight) {
		return maps.get(layer).getURL(x, y, saveWidth, saveHeight);
	}

	public void newMaps(final int width, final int height) {
		maps.clear();
		maps.add(new Board(width, height));
		layer = 0;
	}

	public void loadUrl(String query) {
		maps.get(layer).loadUrl(query);
		notifyToLayerListeners();
	}

	public void loadMap(final Cell[][] map) {
		// width = map[0].length;
		// height = map.length;
		maps.clear();
		maps.add(new Board(map));
		layer = 0;
	}

	public Cell[][] getMap() {
		return maps.get(layer).getMap();
	}

	public void reverse() {
		for (Board map : maps) {
			map.reverse();
		}
		notifyToLayerListeners();
	}

	public void rotate() {
		for (Board map : maps) {
			map.rotate();
		}
		notifyToLayerListeners();
	}

	public int getColumnCount() {
		return maps.get(0).getColumnCount();
	}

//	public int getHeight() {
	public int getRowCount() {
		return maps.get(0).getRowCount();
	}

	public ArrayList<Board> getMaps() {
		return maps;
	}

}
