package zuozuomu.img;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;

import model.Cell;
import controller.BambooController;

public class ImageMaker {
	private BufferedImage leftBamboo, horizontal;
	private BufferedImage leftBambooOriginal, horizontalOriginal;
	private BufferedImage waku, wakuOriginal;
	private BambooController controller;

	private ColorFactory colorFactory;

	public ImageMaker(BambooController controller) {
		this.controller = controller;
		colorFactory = new ColorFactory();
		int size = controller.getSizeOfCell() + 1;
		Image image;
		try {
			leftBamboo = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			leftBambooOriginal = ImageIO.read(getClass().getResource(
					"leftBamboo.png"));
			image = leftBambooOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			leftBamboo.getGraphics().drawImage(image, 0, 0, null);
			horizontal = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			horizontalOriginal = ImageIO.read(getClass().getResource(
					"horizontalBamboo.png"));
			image = horizontalOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			horizontal.getGraphics().drawImage(image, 0, 0, null);

			waku = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
			wakuOriginal = ImageIO
					.read(getClass().getResource("rectangle.png"));
			image = wakuOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			waku.getGraphics().drawImage(image, 0, 0, null);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void drawCursor(Graphics g) {
		g.setColor(Color.RED);
		if (!controller.isRegion()) {
			g.drawRect(controller.getSizeOfCell() * (controller.getX() - 1),
					controller.getSizeOfCell() * (controller.getY() - 1),
					controller.getSizeOfCell(), controller.getSizeOfCell());
		} else if (controller.isRegion()) {
			int minX = Math.min(controller.getX(), controller.getX0());
			int maxX = Math.max(controller.getX(), controller.getX0());
			int minY = Math.min(controller.getY(), controller.getY0());
			int maxY = Math.max(controller.getY(), controller.getY0());
			for (int i = minX - 1; i <= maxX; i++) {
				g.drawLine(controller.getSizeOfCell() * i, controller
						.getSizeOfCell()
						* (minY - 1), controller.getSizeOfCell() * i,
						controller.getSizeOfCell() * maxY);
			}
			for (int j = minY - 1; j <= maxY; j++) {
				g.drawLine(controller.getSizeOfCell() * (minX - 1), controller
						.getSizeOfCell()
						* j, controller.getSizeOfCell() * maxX, controller
						.getSizeOfCell()
						* j);
			}
		}
	}

	/** Ֆʂ */
	public void makeMap(Graphics2D g) {
		makeMap(g, 0, 0, controller.getRowCount(), controller.getColumnCount());
		drawCursor(g);
	}

	/** Ֆʂ */
	public void makeMap(Graphics2D g, int row0, int column0, int rowCount,
			int columnCount) {
		// g.setStroke(new BasicStroke(1.6f));
		// g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
		// RenderingHints.VALUE_ANTIALIAS_ON);
		g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
				RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		Font numberFont = new Font("SansSerif", Font.PLAIN, controller
				.getSizeOfCell() * 4 / 5);
		g.setFont(numberFont);
		/** 1)Ֆʂ𔒂 */
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, controller.getSizeOfCell() * columnCount, controller
				.getSizeOfCell()
				* rowCount);
		for (int layer = 0; layer <= controller.getLayer(); layer++) {
			/** 2)Ֆʂ𔖂 */
			// g.setColor(new Color(1.0f, 1.0f, 1.0f, 0.5f));
			// g.fillRect(0, 0, controller.getSizeOfCell() * columnCount,
			// controller.getSizeOfCell() * rowCount);
			/** 3)C[ɐFς */
			colorFactory.setColor(layer % 4);
			/** 3)Ֆʂ */
			for (int row = 0; row < rowCount; row++) {
				for (int column = 0; column < columnCount; column++) {
					// ̒n_Ɉ̃V}
					drawShima1(g, layer, row + row0, row0, column + column0,
							column0, column * controller.getSizeOfCell(), row
									* controller.getSizeOfCell(), controller
									.getSizeOfCell());
				}
			}
		}
	}

	private void rotateBamboo(Graphics2D g2, double theta, int row, int column) {
		g2.rotate(theta, column * controller.getSizeOfCell()
				+ (controller.getSizeOfCell() + 1.0) / 2.0, row
				* controller.getSizeOfCell()
				+ (controller.getSizeOfCell() + 1.0) / 2.0);
	}

	private void drawShima1(Graphics2D g, int layer, int row, int row0,
			int column, int column0, int x, int y, int size) {
		g.setColor(colorFactory.getDrawColor());
		if (controller.getNumberAt(layer, row, column) == Cell.KARA
				|| controller.getNumberAt(layer, row, column) == Cell.KABE) {
			// Ȃ
		} else {
			if (controller.getNumberAt(layer, row, column) == Cell.ZERO) {
				// 0V}̂Ƃ
				g.drawOval(x + size / 8, y + size / 8, size - size / 4, size
						- size / 4);
				g.drawRect(x, y, size, size);
			} else if (controller.getNumberAt(layer, row, column) == Cell.SHIMA) {
				// V}̂Ƃ
				int center = (controller.getSizeOfCell() + 1 - controller
						.getSizeOfCell() / 8) / 2;
				g.fillRect(x + center, y + center,
						controller.getSizeOfCell() / 8, controller
								.getSizeOfCell() / 8);
				g.drawRect(x, y, size, size);
			} else if (controller.getNumberAt(layer, row, column) == Cell.BATSU) {
				// oĉƂ
				g.drawLine(x + size / 8, y + size / 8, x + size - size / 8, y
						+ size - size / 8);
				g.drawLine(x + size / 8, y + size - size / 8, x + size - size
						/ 8, y + size / 8);
				g.drawRect(x, y, size, size);
			} else if (controller.getNumberAt(layer, row, column) <= -10) {
				// 2P^V}̂Ƃ
				placeNumber(g, row - row0, column - column0, -controller
						.getNumberAt(layer, row, column), size);
				g.drawRect(x, y, size, size);
			} else if (-10 < controller.getNumberAt(layer, row, column)
					&& controller.getNumberAt(layer, row, column) < 0) {
				// 1P^V}̂Ƃ
				placeNumber(g, row - row0, column - column0, -controller
						.getNumberAt(layer, row, column), size);
				g.drawRect(x, y, size, size);
			} else {
				g.setColor(colorFactory.getBambooColor());
				AffineTransform a = g.getTransform();
				switch (controller.getNumberAt(layer, row, column)) {
				case Cell.HIDARI:
					// |
					// g.drawImage(leftBamboo, x, y, Color.GREEN, null);
					g.drawImage(leftBamboo, x, y,
							colorFactory.getBambooColor(), null);
					break;
				case Cell.YOKO:
					g.drawImage(horizontal, x, y,
							colorFactory.getBambooColor(), null);
					break;
				case Cell.MIGI:
					rotateBamboo(g, Math.PI, row - row0, column - column0);
					g.drawImage(leftBamboo, x, y,
							colorFactory.getBambooColor(), null);
					break;
				case Cell.UE:
					rotateBamboo(g, Math.PI * 0.5, row - row0, column - column0);
					g.drawImage(leftBamboo, x, y,
							colorFactory.getBambooColor(), null);
					break;
				case Cell.TATE:
					rotateBamboo(g, Math.PI * 0.5, row - row0, column - column0);
					g.drawImage(horizontal, x, y,
							colorFactory.getBambooColor(), null);
					break;
				case Cell.SHITA:
					rotateBamboo(g, -Math.PI * 0.5, row - row0, column
							- column0);
					g.drawImage(leftBamboo, x, y,
							colorFactory.getBambooColor(), null);
					break;
				case Cell.MASU:
					break;
				}
				g.setTransform(a);
			}
			g.setColor(Color.BLACK);
			g.drawRect(x, y, size, size);
		}
	}

	/**
	 * }Xɐzu
	 * 
	 * @param g
	 * @param r
	 *            Ֆʏ̍sW
	 * @param c
	 *            Ֆʏ̗W
	 * @param number
	 *            `
	 */
	protected void placeNumber(Graphics2D g, int r, int c, int number, int size) {
		placeString(g, r, c, Integer.toString(number), size);
	}

	/**
	 * }X̒ɕzu
	 * 
	 * @param g
	 * @param r
	 *            Ֆʏ̍sW
	 * @param c
	 *            Ֆʏ̗W
	 * @param string
	 *            `
	 */
	// protected void placeString(Graphics2D g, int r, int c, String string, int
	// size) {
	protected void placeString(Graphics2D g, int r, int c, String string,
			int size) {
		drawString(g, c * size + size / 2, r * size + size / 2, string);
	}

	/**
	 * ̍W𒆐SƂĕ`
	 * 
	 * @param g
	 * @param x
	 *            SxW
	 * @param y
	 *            SyW
	 * @param str
	 *            `
	 */
	protected void drawString(Graphics g, int x, int y, String str) {
		FontMetrics metrics = g.getFontMetrics();
		g.drawString(str, x - (metrics.stringWidth(str) - 1) / 2, y
				- metrics.getHeight() / 2 + metrics.getAscent());
	}

	public void write(Graphics2D g, int left, int right, int top, int bottom) {
		writeMap(g, top - 1, left - 1, bottom - top + 1, right - left + 1);
	}

	public void writeMap(Graphics2D g, int row0, int column0, int rowCount,
			int columnCount) {
		/** 1)Ֆʂ𔒂 */
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, controller.getSizeOfCell() * columnCount, controller
				.getSizeOfCell()
				* rowCount);
		for (int layer = 0; layer <= controller.getLayer(); layer++) {
			/** 2)Ֆʂ𔖂 */
			// g.setColor(new Color(1.0f, 1.0f, 1.0f, 0.5f));
			// g.fillRect(0, 0, controller.getSizeOfCell() * columnCount,
			// controller.getSizeOfCell() * rowCount);
			/** 3)C[ɐFς */
			colorFactory.setColor(layer % 4);
			/** 3)Ֆʂ */
			// g.setStroke(new BasicStroke(1.6f));
			g.setStroke(new BasicStroke(1.5f));
//			g.setStroke(new BasicStroke(1.0f));
//			g.setStroke(new BasicStroke(1.1f));
//			g.setStroke(new BasicStroke(1.2f));
//			g.setStroke(new BasicStroke(1.3f));
//			g.setStroke(new BasicStroke(1.4f));
//			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
	//				RenderingHints.VALUE_ANTIALIAS_ON);
			g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
					RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
			Font numberFont = new Font("SansSerif", Font.PLAIN, controller
					.getSizeOfCell() * 4 / 5);
			g.setFont(numberFont);
			for (int row = 0; row < rowCount; row++) {
				for (int column = 0; column < columnCount; column++) {
					// ̒n_Ɉ̃V}
					// drawShima1(g, layer, row + row0, row0, column + column0,
					writeShima(g, layer, row + row0, row0, column + column0,
							column0, column * controller.getSizeOfCell(), row
									* controller.getSizeOfCell(), controller
									.getSizeOfCell());
				}
			}
			// g
			for (int row = 0; row < rowCount; row++) {
				for (int column = 0; column < columnCount; column++) {
					g.drawLine(0, row * controller.getSizeOfCell(), columnCount
							* controller.getSizeOfCell(), row
							* controller.getSizeOfCell());
					g.drawLine(column * controller.getSizeOfCell(), 0, column
							* controller.getSizeOfCell(), rowCount
							* controller.getSizeOfCell());
				}
			}
		}
	}

	private void writeShima(Graphics2D g, int layer, int row, int row0,
			int column, int column0, int x, int y, int size) {
		// g.setColor(colorFactory.getDrawColor());
		g.setColor(Color.BLACK);
		if (controller.getNumberAt(layer, row, column) == Cell.ZERO) {
			// 0V}̂Ƃ
			// g.drawImage(waku, x, y, null);
			g.drawOval(x + size / 8, y + size / 8, size - size / 4, size - size
					/ 4);
			g.drawRect(x, y, size, size);
		} else if (controller.getNumberAt(layer, row, column) == Cell.SHIMA) {
			// V}̂Ƃ
			int center = (controller.getSizeOfCell() + 1 - controller
					.getSizeOfCell() / 8) / 2;
			g.fillRect(x + center, y + center, controller.getSizeOfCell() / 8,
					controller.getSizeOfCell() / 8);
			g.drawRect(x, y, size, size);
		} else if (controller.getNumberAt(layer, row, column) == Cell.BATSU) {
			// oĉƂ
			g.drawLine(x + size / 8, y + size / 8, x + size - size / 8, y
					+ size - size / 8);
			g.drawLine(x + size / 8, y + size - size / 8, x + size - size / 8,
					y + size / 8);
			g.drawRect(x, y, size, size);
		} else if (controller.getNumberAt(layer, row, column) <= -10) {
			// 2P^V}̂Ƃ
			placeNumber(g, row - row0, column - column0, -controller
					.getNumberAt(layer, row, column), size);
			g.drawRect(x, y, size, size);
		} else if (-10 < controller.getNumberAt(layer, row, column)
				&& controller.getNumberAt(layer, row, column) < 0) {
			// 1P^V}̂Ƃ
			placeNumber(g, row - row0, column - column0, -controller
					.getNumberAt(layer, row, column), size);
			g.drawRect(x, y, size, size);
		} else if (controller.getNumberAt(layer, row, column) == Cell.KARA
				|| controller.getNumberAt(layer, row, column) == Cell.KABE) {
			// Ȃ
		} else {
			g.setColor(colorFactory.getBambooColor());
			AffineTransform a = g.getTransform();
			switch (controller.getNumberAt(layer, row, column)) {
			case Cell.HIDARI:
				// |
				g.drawImage(leftBamboo, x, y, null);
				break;
			case Cell.YOKO:
				g.drawImage(horizontal, x, y, null);
				break;
			case Cell.MIGI:
				rotateBamboo(g, Math.PI, row - row0, column - column0);
				g.drawImage(leftBamboo, x, y, null);
				break;
			case Cell.UE:
				rotateBamboo(g, Math.PI * 0.5, row - row0, column - column0);
				g.drawImage(leftBamboo, x, y, null);
				break;
			case Cell.TATE:
				rotateBamboo(g, Math.PI * 0.5, row - row0, column - column0);
				g.drawImage(horizontal, x, y, null);
				break;
			case Cell.SHITA:
				rotateBamboo(g, -Math.PI * 0.5, row - row0, column - column0);
				g.drawImage(leftBamboo, x, y, null);
				break;
			case Cell.MASU:
				break;
			}
			g.setTransform(a);
			g.setColor(Color.BLACK);
			g.drawRect(x, y, size, size);
		}
	}

}
